import processing.core.*; import processing.serial.*; import java.applet.*; import java.awt.*; import java.awt.image.*; import java.awt.event.*; import java.io.*; import java.net.*; import java.text.*; import java.util.*; import java.util.zip.*; public class roombacomm_tst extends PApplet {//
// RoombaComm User Interface
//
//  Tod E. Kurt, tod@todbot.com
//
//

RoombaComm roombaComm = null;

// if you set roombaCommPort to -1, it will do the port picker UI
int roombaCommPort = -1;
//int roombaCommPort = 3;
//int roombaCommPort = 8;
int myframerate = 15;
int sensor_update_every = 5;  // update every 5 frames

// map dir values to roomba radius numbers (absolute value)
int[] dirmap = { 1500, 1000, 500, 250, 125, 60, 30, 15 };
int max_dir  = dirmap.length ;
int [] speedmap = { 100, 200, 300, 400, 500 };
// int [] speedmap = { -500, -400, -300, -200, -100, 0, 100, 200, 300, 400, 500 };
int max_speed = speedmap.length;

float angle_fudge_factor = 0.25f;  // FIXME: why a divide-by-four?
float dist_fudge_factor = 0.25f;

int sensor_framecnt = 0;
int sdistance, sangle;
boolean sensorfault = true;
boolean safetyfault = true;

int c,d;
int octave = 3;  // which octave we're on when playing notes
boolean vacuumOn = false;

float rx,ry,rangle;  // display info for displayed roomba icon

PFont fontA;
PImage rpic;
boolean toggleDraw = false;
float rradius = 20.0f;


// called by processing once on startup
public void setup()
{
  size(450,450);
  background(153); stroke(100); 
  ellipseMode(CENTER_RADIUS);
  rectMode(CENTER);
  smooth();
  framerate(myframerate);

  rpic = loadImage("roomba-sm.png");

  fontA = loadFont("CourierNewPSMT-24.vlw"); 
  textFont(fontA, 12);
  fill(0);

  roombaComm = new RoombaComm(this);
  if( roombaCommPort >= 0 ) {
    if( roombaComm.connect(roombaCommPort) ) {
      println("connected to Roomba!");
      roombaComm.startup();
    }
  }
  
  rx = width/2;
  ry = height/2;
  rangle = 0;
}

// called by Processing 'framerate' times a second
public void draw()
{
  if( roombaComm.connected == false ) { roombaCommSetup(); return; }

  textFont(fontA, 12);
  fill(0);  background(153); stroke(100);

  updateRoombaState();
  
  drawGridlines();
  drawJoystick(); 
  drawStatus();

  drawRoomba();
}

public void updateRoombaState() {
    if( ++sensor_framecnt == sensor_update_every ) {
    sensorfault = ! roombaComm.updateSensors();
    if( ! roombaComm.safetyFault ) {
      if( safetyfault ) {
        roombaComm.reset();
        c = d = 0;
        safetyfault = false;
      } 
    } else {
      safetyfault = true;
    }
    sensor_framecnt=0;
    sdistance = roombaComm.distance();
    sangle    = roombaComm.angle();
    //sangle_dxeg= int((360 * sangle)/(258*PI));
  }
  
  // compute drawing params from roomba sensor data
  // FIXME: for some reason this is screwing up when roomba goes into reverse
  float da = ((2.0f * sangle)/258.0f) * angle_fudge_factor;
  rangle = rangle - da;
  rangle %= TWO_PI;
  float dx = sdistance * sin(rangle) * (0.05f);
  float dy = sdistance * cos(rangle) * (0.05f);
  rx = rx + dx;
  ry = ry - dy;
  //println("sdist:"+sdistance+" sangle:"+sangle+" dx:"+dx+" dy:"+dy+" rangle:"+rangle);
  // torroidial mapping, sir                                                    
  if( rx > width  ) rx = 0; if( rx < 0 ) rx = width;
  if( ry > height ) ry = 0; if( ry < 0 ) ry = height;

}

// called by Processing whenever a key is pressed
// called repeatedly if a key is held down
public void keyPressed()
{
  int note=-1;
  boolean movekey = false;
  if( key == CODED ) {
    if( keyCode == UP ) {
      c++; movekey=true;
    } else if( keyCode == DOWN ) {
      c--; movekey=true;
    } else if( keyCode == LEFT ) {
      d++; movekey=true;
    } else if( keyCode == RIGHT ) {
      d--; movekey=true;
    }
  }
  else if( key == ' ' ) {
    roombaComm.stop();
    c = 0; d = 0;
  }
  else if( key == '/' ) {
    roombaComm.reset();  // in case we pick the roomba up and it shifts to passive mode
    c = 0; d = 0;
    rx = width/2; ry = height/2; rangle = 0;
  }
  else if( key == ',' ) {
    toggleDraw = !toggleDraw;
  }
  else if( key == '\t' ) {
      vacuumOn = !vacuumOn;
      roombaComm.vacuum( vacuumOn );
  }
  else {
    switch( key ) {
      // pseudo-keyboard to play notes on
      case 'a': note = 0;  break; 
      case 'w': note = 1;  break; 
      case 's': note = 2;  break; 
      case 'e': note = 3;  break; 
      case 'd': note = 4;  break; 
      case 'f': note = 5;  break; 
      case 't': note = 6;  break; 
      case 'g': note = 7;  break; 
      case 'y': note = 8;  break; 
      case 'h': note = 9;  break; 
      case 'u': note =10;  break; 
      case 'j': note =11;  break; 
      case 'k': note =12;  break; 
      case 'o': note =13;  break; 
      case 'l': note =14;  break; 
      case 'z': octave--;  break;  // change octaves
      case 'x': octave++;  break;  // change octaves
    }
  }
  
  // we actually hit a note key
  if( note >= 0 ) {
    octave = (octave < 1) ? 1 : (octave > 7) ? 7 : octave;
    roombaComm.playNote( note+(octave*12) + 36, 10);
  }

  if( movekey ) {
    if( c>max_speed ) c=max_speed; if( c<-max_speed ) c=-max_speed;
    if( d>max_dir )   d=max_dir;   if( d<-max_dir )   d=-max_dir;
    
    int ca = abs(c);  int cs = (c>0)?1:(c<0)?-1:0;  // absolute value & sign
    int sp = c*100; // c!=0 ? cs * speedmap[ca-1];
    if( c == 0 && d == 0 ) {
      roombaComm.stop();
    } else if( d==0 ) {
      roombaComm.goStraight( sp );
    } else if( c==0 ) {
      if( d > 0 )
        roombaComm.spinLeft( d*50 );
      else
        roombaComm.spinRight( -d*50 );
    } else {
      if( d>0 )
        roombaComm.drive( sp, dirmap[d-1] );
      else 
        roombaComm.drive( sp, -dirmap[-d-1] );
    }
  }  
}

// nice little grid to help judge onscreen motion
public void drawGridlines()
{
  int gridspacing = 50;
  stroke(120);    strokeWeight(1); 
  for( int i=0; i< 10; i++ ) {
    line(0, i*gridspacing, width, i*gridspacing);
    line(i*gridspacing,0, i*gridspacing, height);
  }
}

// little roomba icon or scanned picture
public void drawRoomba()
{
  translate(rx,ry);
  rotate(rangle);

  if( toggleDraw ) {
    image( rpic, -rradius,-rradius);
  }
  else {
    noStroke();
    fill(45);
    ellipse(0, 0, rradius, rradius);
    fill(70);
    ellipse(0, 0, rradius-3, rradius-3);
    fill(160);
    rect(0,-rradius+12, 4,15);
    fill(100);
    rect(0,rradius/2, 4,4);
  }
}

// joystiq indicating current speed/direction controls
public void drawJoystick()
{
  int jposx0 = width-50, jposy0 = 150; 
  int jposx1 = PApplet.toInt(jposx0 - (d*5)), jposy1 = PApplet.toInt(jposy0 - (c*5));
  fill(160);  noStroke();
  rect(jposx0,jposy0, 50,50);
  stroke(100); strokeWeight(4);   
  line( jposx0, jposy0, jposx1, jposy1 );  
  stroke(200,100,100); strokeWeight(8);
  line( jposx1, jposy1, jposx1, jposy1 );  
  //  line( jposx, jposy, jposx - (d*5), jposy - (c*5) );
}

// status messages  (FIXME: make all these pixel positions relative)
public void drawStatus()
{ 
  textFont(fontA, 10);
  fill(160);  noStroke();
  rect(width-50,40, 90,70);  // sensor background
  rect(width-50,100, 60,40);  // buttons background
  
  fill(0); stroke(100); strokeWeight(5);
  
  text("cliff", width-63, 14);
  if( roombaComm.cliffLeft()       ) stroke(255,0,0); else stroke(100);
  line(width-90,20,width-87,12);  
  if( roombaComm.cliffFrontLeft()  ) stroke(255,0,0); else stroke(100);
  line(width-80,12,width-70,12);
  if( roombaComm.cliffFrontRight() ) stroke(255,0,0); else stroke(100);
  line(width-30,12,width-20,12);
  if( roombaComm.cliffRight()      ) stroke(255,0,0); else stroke(100);
  line(width-10,20,width-13,12);

  text("bump", width-60, 24);
  if( roombaComm.bumpLeft()  ) stroke(255,0,0); else stroke(100);
  line(width-80,25,width-70,22);
  if( roombaComm.bumpRight() ) stroke(255,0,0); else stroke(100);
  line(width-30,22,width-20,25);

  text("wheels", width-67, 44);  
  if( roombaComm.wheelDropLeft()  ) stroke(255,0,0); else stroke(100);
  line(width-80,42,width-70,42);
  if( roombaComm.wheelDropRight() ) stroke(255,0,0); else stroke(100);
  line(width-30,42,width-20,42);
  if( roombaComm.wheelDropCenter()) stroke(255,0,0); else stroke(100);
  line(width-52,32,width-48,32);

  text("dirt", width-60,54);
  boolean dirt = roombaComm.dirt();
  if( dirt ) stroke(0,0,200); else stroke(100);
  line(width-80,52,width-70,52); 
  line(width-30,52,width-20,52); 

  text("p", width-70,100);
  if( roombaComm.powerButton() ) stroke(0,200,0); else stroke(100);
  line(width-70,105, width-70+2,105);
  text("s", width-50,100);
  if( roombaComm.spotButton()  ) stroke(0,200,0); else stroke(100);
  line(width-50,105, width-50+2,105);
  text("c", width-40,100);
  if( roombaComm.cleanButton() ) stroke(0,200,0); else stroke(100);
  line(width-40,105, width-40+2,105);
  text("m", width-30,100);
  if( roombaComm.maxButton()   ) stroke(0,200,0); else stroke(100);
  line(width-30,105, width-30+2,105);

  textFont(fontA, 14);
  text("sensors",       width-77,70 );
  text("buttons",       width-77,90 );

  text("status: "+ (sensorfault?"no roomba":(safetyfault?"safety fault":"OK")), 8,12);

  textFont(fontA, 12);      
  text("use arrow keys to drive",               10,height-48);
  text("space to stop, / to reset roomba",      10,height-38);
  text("comma toggles view, tab toggles vacuum",10,height-28);
  text("asdf... plays notes",                   10,height-18);
  
  text("distance: "+sdistance, width-130, height-18);
  text("   angle: "+sangle,    width-130, height-8);
//  text("angle:"+int(angle*360/TWO_PI), width-90,height-18);
//  text("speed:"+speed, width-90,height-8);

  if( safetyfault ) {
    println("safteyfault!");
    textFont(fontA, 36);
    text("SAFETY FAULT!", 30,height/3);
    textFont(fontA, 24);
    text("reposition Roomba to reset", 40,height/3+20);
  }

}




// FIXME: put this in RoombaComm?
//
// Present the user with a serial port selector
//
public void roombaCommSetup()
{
  int listlen = roombaComm.portlist.length;
  textFont(fontA, 12);
  background(153); fill(0); stroke(100);
  text("click a port:", 10,10);
  for( int i=0; i< listlen; i++ ) {
    text(i+": "+roombaComm.portlist[i], 10,20+10*i);
  }
  int s = (mouseY-15) / 10;
  if( s >= listlen || s < 0 ) return;
  rect(2,15+10*s, 10,8);
  if( mousePressed ) {
    line(10,20+10*s, width-10, 20+10*s);
    if( roombaComm.connect( s ) ) {
      roombaComm.startup();
      roombaComm.playNote( 72, 10);
      roombaCommPort = s;
    }
  }
}


//
// RoombaComm Serial Interface
//
//  Tod E. Kurt, tod@todbot.com
//
//



// -------------------------------------------------------------------------
//
// -------------------------------------------------------------------------
class RoombaComm
{
  PApplet parent;
  Serial port;
  String[] portlist;
  boolean connected = false;
  boolean sensorsValid = false;
  boolean safetyFault = false;
  byte[] sensor_bytes = new byte[26];

  boolean debug = true;

  // constructor
  RoombaComm(PApplet p) {
    portlist = Serial.list();
    if(debug) { 
      for(int i=0;i<portlist.length; i++) { // List all the available serial ports:
          println(i+":"+portlist[i]);
      }
    }
    parent = p;
  }
  
  //
  //  returns true on successful connect, false otherwise
  //
  public boolean connect(int portnum) {
    String portname = portlist[portnum];
    println("connecting to port '"+portname+"'");
    port = new Serial(parent, portname, 57600);
    connected = port != null && port.port != null; // about the only way I know to do this

    sensorsValid = false;
    return connected;
  }
  
  //
  // toggles DD line via serial port DTR
  //
  public void wakeup() {
    // unimplementable until DTR control is available  
    // can probably do:
    port.port.setDTR(false);
    try { Thread.sleep(500); } catch(Exception e) { }
    port.port.setDTR(true);
  }
  
  //
  //
  //
 public void startup() {
    if(debug) println("RoombaComm:startup");
    port.write( START );
    port.write( CONTROL );
    //port.write( FULL );  // dangerous
  }
  
  // FIXME: put this in RoombaComm?
  public void reset()
  {
    if(debug) println("RoombaComm:reset");
    startup();
    updateSensors();
  }

  //
  // move the roomba via the low-level velocity + radius method
  //
  public void drive( int velocity, int radius ) {
    byte cmd[] = { DRIVE, PApplet.toByte(velocity >>> 8), PApplet.toByte(velocity & 0xff), 
                           PApplet.toByte(radius  >>> 8), PApplet.toByte(radius   & 0xff) };
    if(debug) println("RoombaComm:drive: "+hex(cmd[0])+","+hex(cmd[1])+","+hex(cmd[2])+","+hex(cmd[3])+","+hex(cmd[4]));
    port.write(cmd);
  }
  
  //
  // higher-level movement commands 
  // 
  public void stop() {
    if(debug) println("RoombaComm:stop");
    drive( 0, 0 );
  }
  public void spinLeft( int speed ) {
    drive( speed, 1 ); 
  }
  public void spinRight( int speed ) {
    drive( speed, -1 ); 
  }
  public void spinLeft() {
    spinLeft( 100 );  
  }
  public void spinRight() {
    spinRight( 100 );  
  }
  public void goStraight( int speed ) {
    drive( speed, 0x8000 );
  }
  public void goStraight() {
     goStraight( 200 );
  }
  
  // turns on/off the various LEDs
  public void setLEDs( boolean status_green, boolean status_red, 
                boolean spot, boolean clean, boolean max, boolean dirt, 
                int power_color, int power_intensity ) {
    int v = (status_green?0x20:0) | (status_red?0x10:0) | (spot?0x08:0) | (clean?0x04:0) | (max?0x02:0) | (dirt?0x01:0);
    println("leds: "+binary(v));
    byte cmd[] = { LEDS, PApplet.toByte(v), PApplet.toByte(power_color), PApplet.toByte(power_intensity) };
    port.write(cmd);
  }
  
  public void vacuum( boolean state ) {
    if(debug) println("RoombaComm:vacuum: "+state);
    setMotors( state,state,state );
  }
  
  // turns on/off the non-drive motors
  public void setMotors( boolean mainbrush, boolean vacuum, boolean sidebrush ) {
    port.write( MOTORS );
    port.write( PApplet.toByte( (mainbrush?0x04:0) | (vacuum?0x02:0) | (sidebrush?0x01:0) ) );
  }
  
  // plays a note via the hacky method of defining a one-note song & playing it
  // uses up song slot 15
  public void playNote( int note, int duration ) {
    if(debug) println("RoombaComm:playnote: "+note+":"+duration);
    byte cmd[] = { SONG, 15, 1, PApplet.toByte(note), PApplet.toByte(duration),
                    PLAY, 15 };
    port.write(cmd);
  }
  
  //
  // returns true on successful sensor update, false otherwise
  //
  public boolean updateSensors() {
    byte cmd[] = { SENSORS, 0 };
    port.write(cmd);
    sensorsValid = false;  // assume the worst
    int n = port.available();
    if( n >= 26) {                       // there are enough bytes to read
      n = port.readBytes(sensor_bytes);
      if( n==26 ) {                      // did we get enough?
        sensorsValid = true;             // then everything's good, otherwise bad
        computeSafetyFault();
      }
    } else {
      if(debug) println("RoombaComm:updateSensors:only "+n+" bytes available, not updating sensors");
//      println("buffer contains: "+ port.readString() );
    }
    return sensorsValid;
  }
  
  // if true, indicates we had an event that took the Roomba out of safe mode
  // called on every successful updateSensors
  // use the public boolean 'safety_fault' in your code
  public boolean computeSafetyFault() {
    safetyFault = (sensor_bytes[BUMPSWHEELDROPS] & WHEELDROP_MASK) != 0 ||
            sensor_bytes[CLIFFLEFT] == 1  || sensor_bytes[CLIFFFRONTLEFT] == 1 ||
            sensor_bytes[CLIFFRIGHT] == 1  || sensor_bytes[CLIFFFRONTRIGHT] == 1;
    return safetyFault;
  }

  // higher-level sensor access  (FIXME: optimize by accessing sensor_bytes)
  
  public boolean bump() {
    return (sensor_bytes[BUMPSWHEELDROPS] & BUMP_MASK) !=0;
  }  
  public boolean bumpLeft() {
    return (sensor_bytes[BUMPSWHEELDROPS] & BUMPLEFT_MASK) !=0;
  }
  public boolean bumpRight() {
    return (sensor_bytes[BUMPSWHEELDROPS] & BUMPRIGHT_MASK) !=0;
  }
  public boolean wheelDropLeft() {
    return (sensor_bytes[BUMPSWHEELDROPS] & WHEELDROPLEFT_MASK) !=0;
  }
  public boolean wheelDropRight() {
    return (sensor_bytes[BUMPSWHEELDROPS] & WHEELDROPRIGHT_MASK) !=0;
  }
  public boolean wheelDropCenter() {
    return (sensor_bytes[BUMPSWHEELDROPS] & WHEELDROPCENT_MASK) !=0;
  }
  public boolean dirt() {
    int dl = sensor_bytes[DIRTLEFT];
    int dr = sensor_bytes[DIRTRIGHT];
    //if(debug) println("RoombaComm:dirt: dl,dr="+dl+","+dr);
    return (dl > 100) || (dr > 100);
  }

  public boolean cliffLeft() {
    return (sensor_bytes[CLIFFLEFT] != 0);
  }  
  public boolean cliffFrontLeft() {
    return (sensor_bytes[CLIFFFRONTLEFT] != 0);
  }  
  public boolean cliffFrontRight() {
    return (sensor_bytes[CLIFFFRONTRIGHT] != 0);
  }  
  public boolean cliffRight() {
    return sensor_bytes[CLIFFRIGHT] != 0;
  }
  
  public boolean powerButton() {
    return (sensor_bytes[BUTTONS] & POWERBUTTON_MASK) != 0;
  }
  public boolean spotButton() {
    return (sensor_bytes[BUTTONS] & SPOTBUTTON_MASK) != 0;
  }
  public boolean cleanButton() {
    return (sensor_bytes[BUTTONS] & CLEANBUTTON_MASK) != 0;
  }
  public boolean maxButton() {
    return (sensor_bytes[BUTTONS] & MAXBUTTON_MASK) != 0;
  }

  // lower-level sensor access
  
  public int bumps_wheeldrops() {
    return sensor_bytes[BUMPSWHEELDROPS];
  }
  public int wall() {
    return sensor_bytes[WALL];
  }
  public int cliff_left() {
    return sensor_bytes[CLIFFLEFT];
  }
  public int cliff_frontleft() {
    return sensor_bytes[CLIFFFRONTLEFT];
  }
  public int cliff_frontright() {
    return sensor_bytes[CLIFFFRONTRIGHT];
  }
  public int cliff_right() {
    return sensor_bytes[CLIFFRIGHT];
  }
  public int virtual_wall() {
    return sensor_bytes[VIRTUALWALL];
  }
  public int motor_overcurrents() {
    return sensor_bytes[MOTOROVERCURRENTS];
  }
  public int dirt_left() {
    return sensor_bytes[DIRTLEFT];
  }
  public int dirt_right() {
    return sensor_bytes[DIRTRIGHT];
  }

  public int remote_opcode() {
    return sensor_bytes[REMOTEOPCODE];
  }
  public int buttons() {
    return sensor_bytes[BUTTONS];
  }
  public int distance() {
    return sensor_bytes[DISTANCE_HI] << 8 | sensor_bytes[DISTANCE_LO];
  }
  public int angle() {
    return sensor_bytes[DISTANCE_HI] << 8 | sensor_bytes[ANGLE_LO];
  }
  
  public int charging_state() {
    return sensor_bytes[CHARGINGSTATE];
  }
  public int voltage() {
    return sensor_bytes[VOLTAGE_HI] << 8 | sensor_bytes[VOLTAGE_LO];
  }
  public int current() {
    return sensor_bytes[CURRENT_HI] << 8 | sensor_bytes[CURRENT_LO];
  }
  public int temperature() {
    return sensor_bytes[TEMPERATURE];
  }
  public int charge() {
    return sensor_bytes[CHARGE_HI] << 8 | sensor_bytes[CHARGE_LO];
  }
  public int capacity() {
    return sensor_bytes[CAPACITY_HI] << 8 | sensor_bytes[CAPACITY_LO];
  }

    
  // Roomba SCI opcodes
  // the dumb byte...byte() crap is because processing/java bytes are signed
  byte START   =  PApplet.toByte(128);  // 0
  byte BAUD    =  PApplet.toByte(129);  // 1
  byte CONTROL =  PApplet.toByte(130);  // 0
  byte SAFE    =  PApplet.toByte(131);  // 0
  byte FULL    =  PApplet.toByte(132);  // 0
  byte POWER   =  PApplet.toByte(133);  // 0
  byte SPOT    =  PApplet.toByte(134);  // 0
  byte CLEAN   =  PApplet.toByte(135);  // 0
  byte MAX     =  PApplet.toByte(136);  // 0
  byte DRIVE   =  PApplet.toByte(137);  // 4
  byte MOTORS  =  PApplet.toByte(138);  // 1
  byte LEDS    =  PApplet.toByte(139);  // 3
  byte SONG    =  PApplet.toByte(140);  // 2N+2
  byte PLAY    =  PApplet.toByte(141);  // 1
  byte SENSORS =  PApplet.toByte(142);  // 1
  byte DOCK    =  PApplet.toByte(143);  // 0

  // offsets into sensor_bytes data
  byte BUMPSWHEELDROPS     = 0;
  byte WALL                = 1;
  byte CLIFFLEFT           = 2;
  byte CLIFFFRONTLEFT      = 3;
  byte CLIFFFRONTRIGHT     = 4;
  byte CLIFFRIGHT          = 5;
  byte VIRTUALWALL         = 6;
  byte MOTOROVERCURRENTS   = 7;
  byte DIRTLEFT            = 8;
  byte DIRTRIGHT           = 9;
  byte REMOTEOPCODE        = 10;
  byte BUTTONS             = 11;
  byte DISTANCE_HI         = 12;
  byte DISTANCE_LO         = 13;  
  byte ANGLE_HI            = 14;
  byte ANGLE_LO            = 15;
  byte CHARGINGSTATE       = 16;
  byte VOLTAGE_HI          = 17;
  byte VOLTAGE_LO          = 18;  
  byte CURRENT_HI          = 19;
  byte CURRENT_LO          = 20;
  byte TEMPERATURE         = 21;
  byte CHARGE_HI           = 22;
  byte CHARGE_LO           = 23;
  byte CAPACITY_HI         = 24;
  byte CAPACITY_LO         = 25;

  // bitmasks for various thingems
  byte WHEELDROP_MASK      = 0x1C;
  byte BUMP_MASK           = 0x03;
  byte BUMPRIGHT_MASK      = 0x01;
  byte BUMPLEFT_MASK       = 0x02;
  byte WHEELDROPRIGHT_MASK = 0x04;
  byte WHEELDROPLEFT_MASK  = 0x08;
  byte WHEELDROPCENT_MASK  = 0x10;
  
  byte POWERBUTTON_MASK    = 0x08;  
  byte SPOTBUTTON_MASK     = 0x04;  
  byte CLEANBUTTON_MASK    = 0x02;  
  byte MAXBUTTON_MASK      = 0x01;  

}
static public void main(String args[]) {   PApplet.main(new String[] { "roombacomm_tst" });}}